/*
 * Copyright(c) 1997  Unicom Communications, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for commercial and non-commercial purposes is hereby
 * granted provided that this copyright notice appears in all copies.
 * Please refer to the file "LICENSE" for further copyright and licensing
 * information.
 *
 * Unicom Communications, Inc. disclaims all warranties, whether express
 * or implied, with respect to this software including, without limitation,
 * any warranties of merchantability or fitness for a particular purpose
 * or non-infringement. In no event shall Unicom Communications, Inc. be
 * liable for any damages suffered by licensee as a result of using,
 * modifying or distributing this software or its derivatives.
 */


package unicom.servlet.cgi;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import unicom.servlet.cgi.*;


/**
 * CgiServletMain is the main class for launching a Servlet as a CGI
 * program.  This Java program is generally invoked from a script
 * file "servlet.sh", run as a CGI program in response to a web server
 * request.<p>
 *
 * Since Java has no access to environment variables, this class relies
 * on an external script (eg, "servlet.sh") or program to call this Java
 * class  with all of the CGI variables passed as java "-D" flags.  The
 * properties should be named with the CGI variable name prefixed by
 * "http.".  The following are some examples of the CGI property names:<p>
 * <pre>
 * http.QUERY_STRING
 * http.PATH_INFO
 * http.REMOTE_HOST
 * ...
 * </pre><p>
 *
 * Other configuration properties may be passed via a "servlet properties"
 * file.  The System property "servlet.properties" must be set via a
 * "-D" flag to refer to the servlet properties file (eg,
 * "-Dservlet.properties=/usr/local/web-config/servlet.properties").<p>
 *
 * The following is an example of the parameters that may be configured
 * using the servlet properties file:<p>
 * <pre>
 *  ## servlet.debug={true|false}
 *  ##
 *  ##   If servlet.debug is set to 'true', the Content-type of the servlet
 *  ##   return is silently changed to "text/plain", and a running dump
 *  ##   of debugging information is printed, to include stack traces for
 *  ##   any exceptions which occur.
 *  ##
 *  servlet.debug=false
 *
 #  ## server.root=(directory)
 *  ##
 *  ##   Path to the root of the web server administration directory.
 *  ##
 *  server.root=
 *
 *  ## document.root=(directory)
 *  ##
 *  ##   Path to the primary document directory for this web server.
 *  ##
 *  document.root=
 *
 *  ## servlet.timeout=(seconds)
 *  ##
 *  ##   Maximum number of seconds a servlet is allowed to run.  If a servlet
 *  ##   exceeds this time, it is stopped and an error is returned.
 *  ##
 *  servlet.timeout=30
 *
 *  ## servlet.logfile=(file-name)
 *  ##
 *  ##   Name of the file to write log() messages to.
 *  ##
 *  servlet.logfile=servlet.log
 * 
 *  ## servlet.class.{servlet-name}
 *  ##
 *  ##   Contains the mapping of a servlet name to a servlet class.
 *  ##   This is the mechanism by which the class of a servlet is
 *  ##   located.  If the URL "/cgi-bin/servlet.sh/simple" is loaded,
 *  ##   a class mapping for the "simple" servlet must be specified.
 *  ##
 *  ## Examples:
 *  ##
 *  ##   servlet.class.simple=SimpleServlet
 *  ##   servlet.class.redirect=my.pkg.RedirectServlet
 *
 *  ## servlet.args.{*|servlet-name}
 *  ##
 *  ##   Contains parameters to be passed to a servlet durinig servlet
 *  ##   initialization (ie, to the init(ServletConfig) method).  Parameters
 *  ##   defined for "servlet.args.*" are passed to all servlets.
 *  ##
 *  ## Examples:
 *  ##   servlet.args.*=globalparam1=abc,globalparam2=def
 *  ##   servlet.args.MyServlet = myparam=myvalue
 *  ##
 *  servlet.args.*=
 *  </pre><p>
 *
 * CgiServletMain is intended to mimic the functionality of the Java Web
 * Server and Servlet Development Kit---CGI is viewed as just another way
 * to launch a servlet.  No servlet should have to be rewritten just
 * just because it is being run as a CGI program.<p>
 *
 * To give CGI-executed servlets a "look and feel" similar to Java
 * Web Server executed servlets, the functionality of the Invoker
 * servlet is provided.... ie, servlets are accessed using the path
 * "http://.../.../servlet.sh/<servlet-name>/<path-info>".  There should
 * be an executable CGI program named "servlet.sh" in a CGI directory of
 * the web server. The "servlet.sh" executable should exec
 * "java -Dhttp..... unicom.servlet.cgi.CgiServletMain".  This Java
 * program interprets the CGI variables, pulls out the servlet name from
 * the CGI variable PATH_INFO, and instantiates & runs that servlet.
 *
 * @author   Eric Williams
 * @author   Matt Angell
 * @version  $Id: CgiServletMain.java,v 1.4 1997/06/06 20:56:55 ericw Exp $
 */
public class CgiServletMain {

  /**
   * default timeout is 60 seconds
   */
  public static final int TIMEOUT = 60;

  /**
   * Debug mode.  If 'true', the return content-type is changed to
   * "text/plain", and lots of debugging output is rendered.
   */
  protected static boolean debug = false;

  /**
   * Main method used to launch a Java Servlet as a CGI program.
   *
   * @param  args program arguments (not used)
   */
  public static void main(String[] args) {

    Properties servletProperties = new Properties();
    Properties cgiProperties = new Properties();
    int timeout = TIMEOUT;

    // Use the "servlet.properties" property as the servlet configuration
    // file name.  If the "servlet.properties" property is not set, create
    // an empty configuration properties list.
    try {
      String filename = System.getProperty("servlet.properties");
      if (filename != null && !filename.trim().equals("")) {
	CgiUtils.loadProperties(servletProperties, new File(filename));
      }
    } catch (IOException ex) {
      sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
		"servlet.properties is configured incorrectly");
      System.exit(0);
    }

    // Detect whether we're in debug mode or not
    String debugProperty = servletProperties.getProperty("servlet.debug");
    if (debugProperty != null && debugProperty.equals("true")) {
      debug = true;
    }

    // Iterate through the system properties for properties, searching for
    // properties beginning with "http.*" (these are the CGI environment
    // variables).  Strip off the "http.", and add these variables to a
    // list of CGI properties.
    Properties sysProperties = System.getProperties();
    Enumeration sysNames = sysProperties.propertyNames();
    while (sysNames.hasMoreElements()) {
      String name = (String)sysNames.nextElement();
      if (name.startsWith("http.")) {
	String cgiKey = name.substring("http.".length());
	cgiProperties.put(cgiKey, sysProperties.getProperty(name));
      }
    }

    // If we're in debugging mode, then provide an informative dump of
    // all the CGI variables as text/plain
    if (debug) {
      System.out.print("Content-type: text/plain\n\n");
      System.out.print("CGI properties:\n");
      Enumeration cgiNames = cgiProperties.propertyNames();
      while (cgiNames.hasMoreElements()) {
	String key = (String)cgiNames.nextElement();
	System.out.print(key+"="+cgiProperties.getProperty(key)+"\n");
      }
    }

    // Determine if the "servlet.timeout" servlet property is set
    String timeoutString = servletProperties.getProperty("servlet.timeout");
    if (timeoutString != null) {
      try {
	timeout = Integer.parseInt(timeoutString);
      } catch (NumberFormatException ex) {
	sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
		  "servlet.timeout is malformed");
	System.exit(0);
      }
    }

    // Run the bloody servlet in a separate thread so that we can detect
    // if the servlet exceeds its timeout value (if it does, stop it and
    // exit with an error code).
    CgiServletRunner runner =
      new CgiServletRunner(cgiProperties, servletProperties,
			   System.in, System.out);
    runner.setDebug(debug);
    Thread t = new Thread(runner);

    try {
      t.start();
      t.join(timeout*1000);
      if (t.isAlive()) {
	t.stop();
	Thread.sleep(1000);   // give it a chance to stop
	sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
		  "Operation timed out");
      }
    } catch (InterruptedException ex) {
      sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
		"Operation interrupted");
    }

    System.out.flush();
    System.exit(0);
  }


  /**
   * Send an error response to the client.
   *
   * @param code  http error code to return to the client
   * @param msg   error message to return to the client
   */
  protected static void sendError(int code, String msg) {
    System.out.println("Status: "+code+" "+msg);
    System.out.println("Content-type: text/html");
    System.out.println("");
    System.out.println("<H1>"+code+" "+msg+"</H1>");
    System.out.flush();
  }

}
