/*
 * Copyright(c) 1997  Unicom Communications, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for commercial and non-commercial purposes is hereby
 * granted provided that this copyright notice appears in all copies.
 * Please refer to the file "LICENSE" for further copyright and licensing
 * information.
 *
 * Unicom Communications, Inc. disclaims all warranties, whether express
 * or implied, with respect to this software including, without limitation,
 * any warranties of merchantability or fitness for a particular purpose
 * or non-infringement. In no event shall Unicom Communications, Inc. be
 * liable for any damages suffered by licensee as a result of using,
 * modifying or distributing this software or its derivatives.
 */


package unicom.servlet.cgi;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import sun.servlet.http.*;

/**
 * CgiServletRequest is a class that encapsulates most of the translational
 * logic between CGI and Servlet programming.  The CgiServletRequest is
 * passed a Properties object (containing the CGI environment variables)
 * and a POST input stream.
 *
 * @author   Eric Williams
 * @author   Matt Angell
 * @version  $Id: CgiServletRequest.java,v 1.6 1997/06/06 20:56:56 ericw Exp $
 */
public class CgiServletRequest implements HttpServletRequest {

  /** the CGI environment variables, as Java Properties */
  Properties cgiProperties = null;

  /** input stream */
  ServletInputStream in = null;

  /** form data */
  Hashtable params = null;

  /** http headers */
  MimeHeaders headers = null;

  /** Path to document root */
  String documentRoot;

  /** mapping between env var names and HTTP header names */
  static Hashtable httpCgiHeaders = new Hashtable();
  static {
    httpCgiHeaders.put("HTTP_ACCEPT",              "Accept");
    httpCgiHeaders.put("HTTP_ACCEPT_CHARSET",      "Accept-Charset");
    httpCgiHeaders.put("HTTP_ACCEPT_ENCODING",     "Accept-Encoding");
    httpCgiHeaders.put("HTTP_ACCEPT_LANGUAGE",     "Accept-Language");
    httpCgiHeaders.put("HTTP_AUTHORIZATION",       "Authorization");
    httpCgiHeaders.put("HTTP_COOKIE",              "Cookie");
    httpCgiHeaders.put("HTTP_FROM",                "From");
    httpCgiHeaders.put("HTTP_HOST",                "Host");
    httpCgiHeaders.put("HTTP_IF_MODIFIED_SINCE",   "If-Modified-Since");
    httpCgiHeaders.put("HTTP_IF_MATCH",            "If-Match");
    httpCgiHeaders.put("HTTP_IF_NONE_MATCH",       "If-None-Match");
    httpCgiHeaders.put("HTTP_IF_RANGE",            "If-Range");
    httpCgiHeaders.put("HTTP_IF_UNMODIFIED_SINCE", "If-Unmodified-Since");
    httpCgiHeaders.put("HTTP_MAX_FORWARDS",        "Max-Forwards");
    httpCgiHeaders.put("HTTP_PROXY_AUTHORIZATION", "Proxy-Authorization");
    httpCgiHeaders.put("HTTP_RANGE",               "Range");
    httpCgiHeaders.put("HTTP_REFERER",             "Referer");
    httpCgiHeaders.put("HTTP_USER_AGENT",          "User-Agent");
  }

  /**
   * Construct a new HttpServletRequest to handle CGI (GET|POST)
   *
   * @param prop  CGI environment variables as a Properties
   * @param inputStream  input stream for POST data
   */
  public CgiServletRequest(Properties prop, ServletInputStream inputStream,
			   String root) {
    cgiProperties = prop;
    in = inputStream;
    this.documentRoot = root;
  }

  /**
   * @return the CGI variable CONTENT_LENGTH
   */
  public int getContentLength() {
    String len = cgiProperties.getProperty("CONTENT_LENGTH");
    if (len == null || len.trim().equals("")) {
      return 0;
    }
    return Integer.parseInt(len);
  }

  /**
   * @return the CGI variable CONTENT_TYPE
   */
  public String getContentType() {
    return cgiProperties.getProperty("CONTENT_TYPE");
  }

  /**
   * @return the CGI variable SERVER_PROTOCOL
   */
  public String getProtocol() {
    return cgiProperties.getProperty("SERVER_PROTOCOL");
  }

  /**
   * @return the CGI variable SERVER_NAME
   */
  public String getServerName() {
    return cgiProperties.getProperty("SERVER_NAME");
  }

  /**
   * @return the CGI variable SERVER_PORT
   */
  public int getServerPort() {
    return Integer.parseInt(cgiProperties.getProperty("SERVER_PORT"));
  }

  /**
   * @return the CGI variable REMOTE_ADDR
   */
  public String getRemoteAddr() {
    return cgiProperties.getProperty("REMOTE_ADDR");
  }

  /**
   * @return the CGI variable REMOTE_HOST
   */
  public String getRemoteHost() {
    return cgiProperties.getProperty("REMOTE_HOST");
  }

  /**
   * @return the scheme of this request (hard coded to "http", since
   *  there is no CGI-portable way to determine whether it is http or
   *  https)
   */
  public String getScheme() {
    return "http";
  }

  /**
   * Resolves a virtual document path to a real filesystem path.
   * @param vpath  virtual path from document root
   * @return  the full path of the specified virtual path
   */
  public String getRealPath(String vpath) {
    return (documentRoot+vpath);
  }

  /**
   * @return the input stream for this servlet (ie, POST data stream)
   */
  public ServletInputStream getInputStream() {
    return in;
  }

  /**
   * @param name  form field name to search for
   * @return the value associated with the specified form field name
   */
  public String getParameter(String name) {
    if (params == null) {
      initializeParams();
    }
    String[] values = (String[]) params.get(name);
    if (values == null) {
      return null;
    }
    return values[0];
  }

  /**
   * @param name  form field name to search for
   * @return the value associated with the specified form field name,
   *  split into an array of strings (based on the comma as a delimiter)
   */
  public String[] getParameterValues(String name) {
    if (params == null) {
      initializeParams();
    }
    return (String[]) params.get(name);
  }

  /**
   * @return  an enumeration of the form fields passed to this servlet
   */
  public Enumeration getParameterNames() {
    if (params == null) {
      initializeParams();
    }
    return params.keys();
  }

  /**
   * Initialize the params hashtable (by parsing QUERY_STRING if the
   * method is "GET", or by reading the input stream and parsing it if
   * the method is "POST".<p>
   *
   * Only GET query strings and urlencoded POSTs are supported at
   * this time.  multipart/form-data support will be added later.
   */
  private void initializeParams() {
    if (params == null) {
      String method = getMethod();
      if (method.equals("GET")) {
	params = HttpUtils.parseQueryString(getQueryString());
      } else if (method.equals("POST")) {
	if (getContentType().equals("application/x-www-form-urlencoded")) {
	  try {
	    int contentLength = getContentLength();
	    byte b[] = new byte[contentLength];
	    int readCount = getInputStream().read(b);
	    String s = new String(b);
	    params = HttpUtils.parseQueryString(s);
	  } catch (IOException ex) {
	    throw new CgiError(ex.toString());
	  }
	} else {
	  throw new CgiError("unsupported Content-type: "+getContentType());
	}
      } else {  // not a GET or POST
	params = new Hashtable();   // leave empty, but initialized
      }
    }
  }

  /**
   * not implemented.
   */
  public Object getAttribute(String key) {
    return null;
  }

  /**
   * @return the CGI variable REQUEST_METHOD
   */
  public String getMethod() {
    return cgiProperties.getProperty("REQUEST_METHOD");
  }

  /**
   * @return the CGI variable REQUEST_URL
   */
  public String getRequestURI() {
    return cgiProperties.getProperty("REQUEST_URL");
  }

  /**
   * @return the CGI variable SCRIPT_NAME
   */
  public String getServletPath() {
    return cgiProperties.getProperty("SCRIPT_NAME");
  }

  /**
   * @return the CGI variable PATH_INFO
   */
  public String getPathInfo() {
    return cgiProperties.getProperty("PATH_INFO");
  }

  /**
   * @return the CGI variable PATH_TRANSLATED
   */
  public String getPathTranslated() {
    return cgiProperties.getProperty("PATH_TRANSLATED");
  }

  /**
   * @return the CGI variable QUERY_STRING
   */
  public String getQueryString() {
    return cgiProperties.getProperty("QUERY_STRING");
  }

  /**
   * @return the CGI variable REMOTE_USER
   */
  public String getRemoteUser() {
    return cgiProperties.getProperty("REMOTE_USER");
  }

  /**
   * @return the CGI variable AUTH_TYPE
   */
  public String getAuthType() {
    return cgiProperties.getProperty("AUTH_TYPE");
  }

  /**
   * Initialize the 'headers' hashtable by converting the CGI params
   * to http headers.  This assumes Netscape-style CGI, where the CGI
   * variable names for http headers begin with "HTTP_".
   */
  private void initializeHeaders() {
    if (headers == null) {
      headers = new MimeHeaders();
      Enumeration cgiNames = httpCgiHeaders.keys();
      while (cgiNames.hasMoreElements()) {
	String cgiName = (String)cgiNames.nextElement();
	String value = cgiProperties.getProperty(cgiName);
	if (value != null && !value.trim().equals("")) {
	  String httpName = (String)httpCgiHeaders.get(cgiName);
	  headers.putHeader(httpName, value);
	}
      }
    }
  }

  /**
   * @param name  name of HTTP header to retrieve
   * @return  the value associated with the specified HTTP header name
   */
  public String getHeader(String name) {
    if (headers == null) {
      initializeHeaders();
    }
    return headers.getHeader(name);
  }

  /**
   * @param name  name of HTTP header to retrieve
   * @return  the value associated with the specified HTTP header name,
   *  converted to an int
   */
  public int getIntHeader(String name) {
    if (headers == null) {
      initializeHeaders();
    }
    return headers.getIntHeader(name);
  }

  /**
   * @param name  name of HTTP header to retrieve
   * @return  the value associated with the specified HTTP header name,
   *  converted to a date (in 'long' format)
   */
  public long getDateHeader(String name) {
    if (headers == null) {
      initializeHeaders();
    }
    return headers.getDateHeader(name);
  }

  /**
   * @return  an enumeration of the http headers for this request
   */
  public Enumeration getHeaderNames() {
    if (headers == null) {
      initializeHeaders();
    }
    return headers.names();
  }

}
