/*
 * Copyright(c) 1997  Unicom Communications, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for commercial and non-commercial purposes is hereby
 * granted provided that this copyright notice appears in all copies.
 * Please refer to the file "LICENSE" for further copyright and licensing
 * information.
 *
 * Unicom Communications, Inc. disclaims all warranties, whether express
 * or implied, with respect to this software including, without limitation,
 * any warranties of merchantability or fitness for a particular purpose
 * or non-infringement. In no event shall Unicom Communications, Inc. be
 * liable for any damages suffered by licensee as a result of using,
 * modifying or distributing this software or its derivatives.
 */


package unicom.servlet.cgi;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Date;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import sun.servlet.http.*;

/**
 * An http response to a CGI request.
 *
 * @author   Eric Williams
 * @version  $Id: CgiServletResponse.java,v 1.5 1997/06/06 20:56:56 ericw Exp $
 */
public class CgiServletResponse implements HttpServletResponse {

  /** output stream */
  ServletOutputStream out = null;

  /**
   * Have the http headers already been sent?? (Headers are send the
   * first time getOutputStream() is called.)
   */
  boolean sentHeaders = false;

  /**
   * List of HTTP headers that will be sent upon calling 
   * getOutputStream for the first time.
   */
  MimeHeaders headers = new MimeHeaders();


  /**
   * Construct a new http response object on the specified output stream.
   */
  public CgiServletResponse(ServletOutputStream outputStream) {
    out = outputStream;
  }

  /**
   * @return true if the specified header name has already been set;
   *  false otherwise
   */
  public boolean containsHeader(String name) {
    return headers.containsHeader(name);
  }
     
  /**
   * Set the content length of this response (optional, defaults to no
   * Content-length specified).
   */
  public void setContentLength(int len) {
    headers.putIntHeader("Content-length", len);
  }

  /**
   * Set the mime type of this response (not optional, but defaults to
   * "text/html" if not set).
   * @param type  mime content type
   */
  public void setContentType(String type) {
    headers.putHeader("Content-type", type);
  }

  /**
   * Return the output stream for this http request.  If the "Status" has
   * not been set manually, it defaults to SC_OK.  If the "Content-type"
   * has not been set manually or via setContentType, then it defaults to
   * "text/html".  <i>Note: the first time this method is called, it
   * writes the http status, message, and any other headers that have
   * been set.  Status, content type & length, and http headers should be
   * set prior to calling this method.</i><p>
   *
   * @return  the servlet output stream for writing a response
   */
  public ServletOutputStream getOutputStream() throws IOException {
    if (!sentHeaders) {
      if (!headers.containsHeader("Status")) {
	headers.putHeader("Status", SC_OK+" OK");
      }
      if (!headers.containsHeader("Content-type")) {
	headers.putHeader("Content-type", "text/html");  //default
      }
      headers.write(out);
      //out.println("");
      sentHeaders = true;
    }
    return out;
  }

  /**
   * Set the http "Status:" code and message for this http response.
   * <i>NOTE: MUST BE CALLED PRIOR TO getOutputStream().</i><p>
   *
   * @param code  http status code
   * @param msg   http status message
   * @see HttpServletResponse
   */
  public void setStatus(int code, String msg) {
    if (headers.containsHeader("Status")) {
      headers.removeHeader("Status");
    }
    headers.putHeader("Status", code+" "+msg);
  }

  /**
   * Set the http "Status:" code for this http response.
   * <i>NOTE: MUST BE CALLED PRIOR TO getOutputStream().</i><p>
   *
   * @param code  http status code
   * @see HttpServletResponse
   */
  public void setStatus(int code) {
    setStatus(code, "");
  }

  /**
   * Sets an HTTP header.
   * <i>NOTE: MUST BE CALLED PRIOR TO getOutputStream().</i><p>
   *
   * @param name  the Header name
   * @param value the header value
   */
  public void setHeader(String name, String value) {
    headers.putHeader(name,value);
  }

  /**
   * Sets an HTTP header. All headers will be collected, and then output
   * upon the first call to getOutputStream
   * <i>NOTE: MUST BE CALLED PRIOR TO getOutputStream().</i><p>
   *
   * @param name  HTTP header name
   * @param value HTTP header value
   */
  public void setIntHeader(String name, int value) {
    headers.putIntHeader(name, value);
  }

  /**
   * Sets an HTTP header. All headers will be collected, and then output
   * upon the first call to getOutputStream().  Date format will be output
   * in the form "Sat Aug 12 02:30:00 PDT 1995".
   * <i>NOTE: MUST BE CALLED PRIOR TO getOutputStream().</i><p>
   *
   * @param name  HTTP Header name
   * @param value HTTP header value in miliseconds since the epoch.
   */
  public void setDateHeader(String name, long value) {
    headers.putDateHeader(name, value);
  }

  /**
   * Send an error code to the client's browser.
   * @param code  http error code
   * @param msg   error message text
   */
  public void sendError(int code, String msg) throws IOException {
    headers.clear();
    headers.putHeader("Status", code + " " + msg);
    headers.putHeader("Content-type", "text/html");
    headers.write(out);
    //out.println("");
    out.println("<H1>"+code+" "+msg+"</H1>");
    sentHeaders = true;
  }

  /**
   * Send an error code to the client's browser.
   * @param code  http error code
   */
  public void sendError(int code) throws IOException {
    sendError(code, "");
  }

  /**
   * Redirect the client's browser to a different URL.
   * @param location  url to redirect the client's browser to
   */
  public void sendRedirect(String location) throws IOException {
    headers.clear();
    headers.putHeader("Status", SC_MOVED_TEMPORARILY+" Found");
    headers.putHeader("Location", location);
    headers.write(out);
    //out.println("");
    sentHeaders = true;
  }

  /**
   * Flushes all output of this http response.  (Note: the call to
   * getOutputStream() is important, because the first call to
   * getOutputStream() writes the http headers for the response.)
   */
  public void flush() throws IOException {
    getOutputStream().flush();
  }

}
