/*
 * Copyright(c) 1997  Unicom Communications, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for commercial and non-commercial purposes is hereby
 * granted provided that this copyright notice appears in all copies.
 * Please refer to the file "LICENSE" for further copyright and licensing
 * information.
 *
 * Unicom Communications, Inc. disclaims all warranties, whether express
 * or implied, with respect to this software including, without limitation,
 * any warranties of merchantability or fitness for a particular purpose
 * or non-infringement. In no event shall Unicom Communications, Inc. be
 * liable for any damages suffered by licensee as a result of using,
 * modifying or distributing this software or its derivatives.
 */


package unicom.servlet.cgi;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import unicom.servlet.*;
import unicom.servlet.cgi.*;

/**
 * CgiServletRunner is part of the CGI to servlet translation layer.
 * This "runner" class accepts a set of properties (identical to the
 * CGI environment variables), an input stream, and an output stream.
 * The class is responsible for instantiating the servlet class,
 * and running the servlet.  No fancy classloaders are used... just
 * Class.forName().<p>
 * 
 * CGI variables are passed to the CgiServletRunner via a Properties
 * object; the keys of the properties object are spelled exactly like
 * traditional CGI environment variables.<p>
 *
 * @author   Eric Williams
 * @author   Matt Angell
 * @version  $Id: CgiServletRunner.java,v 1.5 1997/06/06 20:56:57 ericw Exp $
 */
public class CgiServletRunner implements Runnable {

  /** debug mode */
  boolean debug = false;

  /** equivalent to CGI environment variables */
  Properties cgiProperties = null;

  /** servlet configuration file */
  Properties servletProperties = null;

  /** input stream (for POST or PUT info) */
  ServletFilterInputStream in = null;

  /** output stream (for servlet response) */
  ServletFilterOutputStream out = null;

  /**
   * Construct a new
   * @param  cgiProp  CGI variables
   * @param  serveltProp  servlet configuration properties
   * @param  input  input stream
   * @param  output  output stream
   */
  public CgiServletRunner(Properties cgiProp, Properties servletProp,
			  InputStream input, OutputStream output) {
    cgiProperties = cgiProp;
    servletProperties = servletProp;
    in = new ServletFilterInputStream(input);
    out = new ServletFilterOutputStream(output);
  }

  /**
   * Set the debug mode for this servlet runner.
   * @param debug  debug mode for this servlet runner
   */
  public void setDebug(boolean debug) {
    this.debug = debug;
  }

  /**
   * Construct and run the specified servlet.  The servlet class name is
   * contained in the PATH_INFO variable.  For example, PATH_INFO might
   * be set to "/DbServlet/foo/bar".  In this case, the name of the servlet
   * is set to "DbServlet" (and the PATH_INFO is set to "/foo/bar", since
   * the intent of PATH_INFO is to hold *extra* path information after the
   * executable content name).
   */
  public void run() {
    String docRoot = servletProperties.getProperty("document.root", "");
    String serverRoot = servletProperties.getProperty("server.root", "");

    // Each servlet has a request and a response
    CgiServletRequest request = 
      new CgiServletRequest(cgiProperties, in, docRoot);
    CgiServletResponse response = new CgiServletResponse(out);
    Properties servletConfig = new Properties();

    // Strip the leading "/" off PATH_INFO to get the servlet name
    // the serlvet name is then used to look up the servlet parameter
    // 'servlet.class.(servletName) to find the acutal class name to load
    // and run. This procuedure emulates the serlvet alising done by the
    // java web server
    String servletName = cgiProperties.getProperty("PATH_INFO");
    if (servletName.startsWith("/")) {
      servletName = servletName.substring(1);
    }

    // Strip any trailing "/...." path off the servlet name, and use
    // the trailing as the new PATH_INFO.
    int loc = servletName.indexOf("/");
    if (loc == -1) {
      cgiProperties.remove("PATH_INFO");
    } else {
      String newPathInfo = servletName.substring(loc);
      cgiProperties.put("PATH_INFO", newPathInfo);
      servletName = servletName.substring(0, loc);
    }

    try {
      // Load the configuration properties global to all servlets
      String globalProperties =
	servletProperties.getProperty("servlet.args.*");
      if (globalProperties != null) {
	CgiUtils.loadProperties(servletConfig, globalProperties);
      }

      // Load the configuration properties specific to this servlet
      String specificProperties =
	servletProperties.getProperty("servlet.args."+servletName);
      if (specificProperties != null) {
	CgiUtils.loadProperties(servletConfig, specificProperties);
      }

      // Detect the log file configuration
      String logFileName = servletProperties.getProperty("servlet.logfile");

      // Find the real class name for the serlvet from the servlet parameters
      String servletClassName = 
	servletProperties.getProperty("servlet.class."+servletName);      
     
      // Test if the servletClassName is valid
      if (servletClassName == null) {
	throw new ClassNotFoundException("class not found: "+servletClassName);
      }
	
      // Load the servlet class
      Class c = Class.forName(servletClassName);

      // Instantiate the servlet object
      Servlet servlet = (Servlet)c.newInstance();

      // Configure and run the new servlet
      CgiServletContext context = 
	new CgiServletContext(logFileName, serverRoot);
      CgiServletConfig config =	new CgiServletConfig(context, servletConfig);
      servlet.init(config);
      servlet.service(request, response);

      // Flush the servlet response
      response.flush();

    } catch (ThreadDeath td) {
      throw td;    // ignore, and send it on
    } catch (ClassNotFoundException ee) {
      error(response, ee, HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
	    "Servlet not found: "+servletName);
    } catch (ClassCastException ee) {
      error(response, ee, HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
	    "Not a servlet: "+servletName);
    } catch (Throwable t) {
      error(response, t, HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
	    t.getClass().getName()+": "+t.getMessage());
    }

  }

  /**
   * Convenience error handling routine for returning an error to
   * the client in response to an exception that was thrown in the
   * servlet.
   *
   * @param response  http response object
   * @param t         the exception that spawned this error() call
   * @param code      http error code
   * @param msg       http error message
   */
  protected void error(CgiServletResponse response, Throwable t,
		       int code, String msg)
    {
      try {
	if (debug) {
	  t.printStackTrace(System.out);
	  System.out.flush();
	}
	response.sendError(code, msg);
      } catch (IOException ex) {
	// something is very very badly hosed.... don't go silently
	throw new CgiError(t.toString());
      }
    }

}
